# barbacompta - accounting for dummies
# Copyright (C) 2010-2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import csv

from django.core.exceptions import PermissionDenied
from django.http import HttpResponse
from django.utils.six import text_type


def export_as_csv(modeladmin, request, queryset):
    """
    Generic csv export admin action.
    """
    if not request.user.is_staff:
        raise PermissionDenied
    opts = modeladmin.model._meta
    response = HttpResponse(content_type="text/csv")
    response["Content-Disposition"] = "attachment; filename=%s.csv" % text_type(opts).replace(".", "_")
    writer = csv.writer(response)
    field_names = [field.name for field in opts.fields]
    m2m_field_names = [m2m_field.name for m2m_field in opts.many_to_many]
    # Write a first row with header information
    writer.writerow(field_names + m2m_field_names)
    # Write data rows
    for obj in queryset:
        values = [text_type(getattr(obj, field)) for field in field_names]
        for m2m_field in m2m_field_names:
            value = getattr(obj, m2m_field)
            value = u",".join(map(text_type, value.all()))
            values.append(text_type(value))
        writer.writerow(map(lambda x: text_type.encode(x, "utf8"), values))
    return response


export_as_csv.short_description = "Export selected objects as csv file"
