# barbacompta - accounting for dummies
# Copyright (C) 2010-2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import cgi
import json
import os.path

from django import http
from django.shortcuts import render
from django.template.loader import get_template
from django.conf import settings
from weasyprint import HTML

from . import models


def render_to_pdf(template_src, context_dict):
    template = get_template(template_src)
    html = HTML(string=template.render(context_dict))
    try:
        pdf = html.write_pdf()
        if hasattr(settings, 'FACTURE_DIR'):
            facture = context_dict['facture']
            filename = os.path.join(
                settings.FACTURE_DIR,
                '%s-%s.pdf' % (facture.code(), facture.contrat.client.nom.encode('utf8')),
            )
            with open(filename, 'wb') as fd:
                fd.write(pdf)
        return http.HttpResponse(pdf, content_type='application/pdf')

    except IOError:
        return http.HttpResponse('We had some errors<pre>%s</pre>' % cgi.escape(html))


def facture(request, facture):
    context = {'facture': models.Facture.objects.get(id=facture)}
    response = render(request, 'facture.html', context=context)
    return response


def facture_pdf(request, facture):
    context = {'facture': models.Facture.objects.get(id=facture), 'base_uri': request.build_absolute_uri('/')}
    response = render_to_pdf('facture.html', context)
    return response


def api_references(request):
    data = []
    for contract in (
        models.Contrat.objects.exclude(public_description='')
        .exclude(client__picture__isnull=True)
        .exclude(client__picture='')
    ):
        data.append(
            {
                'title': contract.intitule,
                'description': contract.public_description,
                'client': contract.client.nom,
                'logo': request.build_absolute_uri(contract.client.picture.url),
            }
        )
        if contract.image:
            data[-1]['image'] = request.build_absolute_uri(contract.image.url)
    return http.HttpResponse(json.dumps({'data': data}), content_type='application/json')
